@@ -1,5 +1,6 @@
 /*
- *  Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
+ *  Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
+ *  Copyright (C) 2004 Free Software Foundation
  *
  *  This file is part of GNUTLS.
  *
@@ -37,23 +38,31 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.8.6"
-
-/* For backwards compatibility */
-#include <gnutls/compat4.h>
+#define LIBGNUTLS_VERSION "1.0.13"
 
+#ifndef HAVE_SIZE_T
+# define HAVE_SIZE_T
 #include <sys/types.h>
-#include <time.h>
+#endif
 
-#define GNUTLS_CIPHER_AES_128_CBC GNUTLS_CIPHER_RIJNDAEL_128_CBC
-#define GNUTLS_CIPHER_AES_256_CBC GNUTLS_CIPHER_RIJNDAEL_256_CBC
-#define GNUTLS_CIPHER_RIJNDAEL_CBC GNUTLS_CIPHER_RIJNDAEL_128_CBC
+#ifndef HAVE_SSIZE_T
+# define HAVE_SSIZE_T
+
+#endif
+
+#ifndef HAVE_TIME_T
+# define HAVE_TIME_T
+#include <time.h>
+#endif
+
+#define GNUTLS_CIPHER_RIJNDAEL_128_CBC GNUTLS_CIPHER_AES_128_CBC
+#define GNUTLS_CIPHER_RIJNDAEL_256_CBC GNUTLS_CIPHER_AES_256_CBC
+#define GNUTLS_CIPHER_RIJNDAEL_CBC GNUTLS_CIPHER_AES_128_CBC
 #define GNUTLS_CIPHER_ARCFOUR GNUTLS_CIPHER_ARCFOUR_128
 
 typedef enum gnutls_cipher_algorithm { GNUTLS_CIPHER_NULL=1, 
-	GNUTLS_CIPHER_ARCFOUR_128, GNUTLS_CIPHER_3DES_CBC, GNUTLS_CIPHER_RIJNDAEL_128_CBC, 
-	GNUTLS_CIPHER_TWOFISH_128_CBC, GNUTLS_CIPHER_RIJNDAEL_256_CBC,
-	GNUTLS_CIPHER_ARCFOUR_40
+	GNUTLS_CIPHER_ARCFOUR_128, GNUTLS_CIPHER_3DES_CBC, GNUTLS_CIPHER_AES_128_CBC, 
+	GNUTLS_CIPHER_AES_256_CBC, GNUTLS_CIPHER_ARCFOUR_40
 } gnutls_cipher_algorithm;
 
 typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS, 
@@ -61,21 +70,30 @@ typedef enum gnutls_kx_algorithm { GNUTLS_KX_RSA=1, GNUTLS_KX_DHE_DSS,
 	GNUTLS_KX_RSA_EXPORT, GNUTLS_KX_SRP_RSA, GNUTLS_KX_SRP_DSS
 } gnutls_kx_algorithm;
 
+typedef enum gnutls_params_type { GNUTLS_PARAMS_RSA_EXPORT=1,
+	GNUTLS_PARAMS_DH 
+} gnutls_params_type;
+
 typedef enum gnutls_credentials_type { GNUTLS_CRD_CERTIFICATE=1, GNUTLS_CRD_ANON, GNUTLS_CRD_SRP } gnutls_credentials_type;
 
-typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, GNUTLS_MAC_MD5, GNUTLS_MAC_SHA } gnutls_mac_algorithm;
+typedef enum gnutls_mac_algorithm { GNUTLS_MAC_NULL=1, 
+	GNUTLS_MAC_MD5, GNUTLS_MAC_SHA, GNUTLS_MAC_RMD160
+} gnutls_mac_algorithm;
 
 /* The enumerations here should have the same value with gnutls_mac_algorithm.
  */
-typedef enum gnutls_digest_algorithm { GNUTLS_DIG_NULL=1, GNUTLS_DIG_MD5, GNUTLS_DIG_SHA } gnutls_digest_algorithm;
+typedef enum gnutls_digest_algorithm { GNUTLS_DIG_NULL=1, GNUTLS_DIG_MD5, 
+	GNUTLS_DIG_SHA, GNUTLS_DIG_RIPEMD160 
+} gnutls_digest_algorithm;
 
 /* exported for other gnutls headers. This is the maximum number
  * of algorithms (ciphers, kx or macs). 
  */
-#define GNUTLS_MAX_ALGORITHM_NUM 10
+#define GNUTLS_MAX_ALGORITHM_NUM 16
 
+#define GNUTLS_COMP_ZLIB GNUTLS_COMP_DEFLATE
 typedef enum gnutls_compression_method { GNUTLS_COMP_NULL=1, 
-	GNUTLS_COMP_ZLIB, 
+	GNUTLS_COMP_DEFLATE, 
 	GNUTLS_COMP_LZO /* only available if gnutls-extra has been initialized 
                          */
 } gnutls_compression_method;
@@ -84,8 +102,9 @@ typedef enum gnutls_connection_end { GNUTLS_SERVER=1, GNUTLS_CLIENT } gnutls_con
 
 typedef enum gnutls_alert_level { GNUTLS_AL_WARNING=1, GNUTLS_AL_FATAL } gnutls_alert_level;
 
-typedef enum gnutls_alert_description { GNUTLS_A_CLOSE_NOTIFY, GNUTLS_A_UNEXPECTED_MESSAGE=10, GNUTLS_A_BAD_RECORD_MAC=20,
-	GNUTLS_A_DECRYPTION_FAILED, GNUTLS_A_RECORD_OVERFLOW,  GNUTLS_A_DECOMPRESSION_FAILURE=30,
+typedef enum gnutls_alert_description { GNUTLS_A_CLOSE_NOTIFY, 
+	GNUTLS_A_UNEXPECTED_MESSAGE=10, GNUTLS_A_BAD_RECORD_MAC=20,
+	GNUTLS_A_DECRYPTION_FAILED, GNUTLS_A_RECORD_OVERFLOW, GNUTLS_A_DECOMPRESSION_FAILURE=30,
 	GNUTLS_A_HANDSHAKE_FAILURE=40, GNUTLS_A_SSL3_NO_CERTIFICATE=41,
 	GNUTLS_A_BAD_CERTIFICATE=42, GNUTLS_A_UNSUPPORTED_CERTIFICATE,
 	GNUTLS_A_CERTIFICATE_REVOKED, GNUTLS_A_CERTIFICATE_EXPIRED, GNUTLS_A_CERTIFICATE_UNKNOWN,
@@ -93,14 +112,34 @@ typedef enum gnutls_alert_description { GNUTLS_A_CLOSE_NOTIFY, GNUTLS_A_UNEXPECT
 	GNUTLS_A_DECRYPT_ERROR, GNUTLS_A_EXPORT_RESTRICTION=60, GNUTLS_A_PROTOCOL_VERSION=70,
 	GNUTLS_A_INSUFFICIENT_SECURITY, GNUTLS_A_INTERNAL_ERROR=80, GNUTLS_A_USER_CANCELED=90,
 	GNUTLS_A_NO_RENEGOTIATION=100, GNUTLS_A_UNSUPPORTED_EXTENSION=110,
-	GNUTLS_A_CERTIFICATE_UNOBTAINABLE=111, GNUTLS_A_UNRECOGNIZED_NAME=112
+	GNUTLS_A_CERTIFICATE_UNOBTAINABLE=111, GNUTLS_A_UNRECOGNIZED_NAME=112,
+	GNUTLS_A_UNKNOWN_SRP_USERNAME=120, GNUTLS_A_MISSING_SRP_USERNAME=121
 } gnutls_alert_description;
 
+typedef enum gnutls_handshake_description { GNUTLS_HANDSHAKE_HELLO_REQUEST, 
+	GNUTLS_HANDSHAKE_CLIENT_HELLO, GNUTLS_HANDSHAKE_SERVER_HELLO,
+	GNUTLS_HANDSHAKE_CERTIFICATE_PKT=11, GNUTLS_HANDSHAKE_SERVER_KEY_EXCHANGE,
+	GNUTLS_HANDSHAKE_CERTIFICATE_REQUEST, GNUTLS_HANDSHAKE_SERVER_HELLO_DONE,
+	GNUTLS_HANDSHAKE_CERTIFICATE_VERIFY, GNUTLS_HANDSHAKE_CLIENT_KEY_EXCHANGE,
+	GNUTLS_HANDSHAKE_FINISHED=20 
+} gnutls_handshake_description;
+
+/* Note that the status bits have different meanings
+ * in openpgp keys and x.509 certificate verification.
+ */
 typedef enum gnutls_certificate_status { 
-	GNUTLS_CERT_NOT_TRUSTED=2, 
-	GNUTLS_CERT_INVALID=4, 
-	GNUTLS_CERT_CORRUPTED=16,
-	GNUTLS_CERT_REVOKED=32
+	GNUTLS_CERT_INVALID=2, /* will be set if the certificate
+				* was not verified.
+				*/
+	GNUTLS_CERT_REVOKED=32, /* in X.509 this will be set only if CRLs are checked
+				 */
+
+	/* Those are extra information about the verification
+	 * process. Will be set only if the certificate was 
+	 * not verified.
+	 */
+	GNUTLS_CERT_SIGNER_NOT_FOUND=64,
+	GNUTLS_CERT_SIGNER_NOT_CA=128
 } gnutls_certificate_status;
 
 typedef enum gnutls_certificate_request { GNUTLS_CERT_IGNORE, GNUTLS_CERT_REQUEST=1, GNUTLS_CERT_REQUIRE } gnutls_certificate_request;
@@ -111,21 +150,33 @@ typedef enum gnutls_openpgp_key_status { GNUTLS_OPENPGP_KEY,
 
 typedef enum gnutls_close_request { GNUTLS_SHUT_RDWR=0, GNUTLS_SHUT_WR=1 } gnutls_close_request;
 
-typedef enum gnutls_protocol_version { GNUTLS_SSL3=1, GNUTLS_TLS1 } gnutls_protocol_version;
+#define GNUTLS_TLS1 GNUTLS_TLS1_0
+typedef enum gnutls_protocol_version { GNUTLS_SSL3=1, GNUTLS_TLS1_0,
+	GNUTLS_TLS1_1 } gnutls_protocol_version;
 
 typedef enum gnutls_certificate_type { GNUTLS_CRT_X509=1, GNUTLS_CRT_OPENPGP 
 } gnutls_certificate_type;
 
-typedef enum gnutls_x509_certificate_format { GNUTLS_X509_FMT_DER, 
-	GNUTLS_X509_FMT_PEM } gnutls_x509_certificate_format;
+typedef enum gnutls_x509_crt_fmt { GNUTLS_X509_FMT_DER, 
+	GNUTLS_X509_FMT_PEM } gnutls_x509_crt_fmt;
 
-typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA
+typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA,
+	GNUTLS_PK_UNKNOWN = 0xff
 } gnutls_pk_algorithm;
 
+const char *gnutls_pk_algorithm_get_name( gnutls_pk_algorithm algorithm);
+
+typedef enum gnutls_sign_algorithm { GNUTLS_SIGN_RSA_SHA = 1, GNUTLS_SIGN_DSA_SHA,
+	GNUTLS_SIGN_RSA_MD5, GNUTLS_SIGN_RSA_MD2,
+	GNUTLS_SIGN_UNKNOWN = 0xff
+} gnutls_sign_algorithm;
+
+const char *gnutls_sign_algorithm_get_name( gnutls_sign_algorithm algorithm);
+
 /* If you want to change this, then also change the 
  * define in gnutls_int.h, and recompile.
  */
-typedef int gnutls_transport_ptr;
+typedef void * gnutls_transport_ptr;
 
 struct gnutls_session_int;
 typedef struct gnutls_session_int* gnutls_session;
@@ -141,6 +192,12 @@ typedef struct {
 	unsigned int size;
 } gnutls_datum;
 
+
+typedef struct {
+	const unsigned char * data;
+	unsigned int size;
+} gnutls_const_datum;
+
 /* internal functions */
 
 int gnutls_init(gnutls_session * session, gnutls_connection_end con_end);
@@ -183,8 +240,8 @@ const char* gnutls_strerror( int error);
 /* Semi-internal functions.
  */
 void gnutls_handshake_set_private_extensions(gnutls_session session, int allow);
-void gnutls_record_set_cbc_protection(gnutls_session session, int prot);
-void gnutls_handshake_set_rsa_pms_check(gnutls_session session, int check);
+gnutls_handshake_description gnutls_handshake_get_last_out( gnutls_session session);
+gnutls_handshake_description gnutls_handshake_get_last_in( gnutls_session session);
 
 /* Record layer functions.
  */
@@ -205,12 +262,13 @@ size_t gnutls_record_check_pending(gnutls_session session);
 typedef enum gnutls_server_name_type { GNUTLS_NAME_DNS=1 
 } gnutls_server_name_type;
 
-int gnutls_get_server_name(gnutls_session session, void* data, int* data_length,
-				       int *type, int index);
+int gnutls_server_name_set(gnutls_session session,
+	gnutls_server_name_type type,
+	const void *name, size_t name_length);
 
-int gnutls_set_server_name(gnutls_session session,
-			       gnutls_server_name_type type,
-			       const void *name, int name_length);
+int gnutls_server_name_get(gnutls_session session, 
+	void *data, size_t *data_length,
+	unsigned int * type, int unsigned indx);
 
 /* functions to set priority of cipher suites 
  */
@@ -238,11 +296,11 @@ const char *gnutls_protocol_get_name(gnutls_protocol_version version);
 
 /* get/set session 
  */
-int gnutls_session_set_data( gnutls_session session, const void* session_data, int session_data_size);
-int gnutls_session_get_data( gnutls_session session, void* session_data, int *session_data_size);
+int gnutls_session_set_data( gnutls_session session, const void* session_data, size_t session_data_size);
+int gnutls_session_get_data( gnutls_session session, void* session_data, size_t *session_data_size);
 /* returns the session ID */
 #define GNUTLS_MAX_SESSION_ID 32
-int gnutls_session_get_id( gnutls_session session, void* session_id, int *session_id_size);
+int gnutls_session_get_id( gnutls_session session, void* session_id, size_t *session_id_size);
 
 /* checks if this session is a resumed one 
  */
@@ -275,6 +333,7 @@ int gnutls_credentials_clear( gnutls_session session);
 /* cred is a structure defined by the kx algorithm 
  */
 int gnutls_credentials_set( gnutls_session, gnutls_credentials_type type, void* cred);
+#define gnutls_cred_set	gnutls_credentials_set
 
 /* Credential structures for SRP - used in gnutls_credentials_set(); */
 
@@ -291,8 +350,8 @@ int gnutls_anon_allocate_server_credentials( gnutls_anon_server_credentials *sc)
 
 void gnutls_anon_set_server_dh_params( gnutls_anon_server_credentials res, gnutls_dh_params dh_params);
 
-void gnutls_anon_free_client_credentials( gnutls_anon_server_credentials sc);
-int gnutls_anon_allocate_client_credentials( gnutls_anon_server_credentials *sc);
+void gnutls_anon_free_client_credentials( gnutls_anon_client_credentials sc);
+int gnutls_anon_allocate_client_credentials( gnutls_anon_client_credentials *sc);
 
 /* CERTFILE is an x509 certificate in PEM form.
  * KEYFILE is a pkcs-1 private key in PEM form (for RSA keys).
@@ -300,30 +359,60 @@ int gnutls_anon_allocate_client_credentials( gnutls_anon_server_credentials *sc)
 void gnutls_certificate_free_credentials( gnutls_certificate_credentials sc);
 int gnutls_certificate_allocate_credentials( gnutls_certificate_credentials *sc);
 
-int gnutls_certificate_set_dh_params(gnutls_certificate_credentials res, gnutls_dh_params);
-int gnutls_certificate_set_rsa_params(gnutls_certificate_credentials res, gnutls_rsa_params rsa_params);
+void gnutls_certificate_free_keys(gnutls_certificate_credentials sc);
+void gnutls_certificate_free_cas(gnutls_certificate_credentials sc);
+void gnutls_certificate_free_ca_names(gnutls_certificate_credentials sc);
+void gnutls_certificate_free_crls(gnutls_certificate_credentials sc);
+
+void gnutls_certificate_set_dh_params(gnutls_certificate_credentials res, gnutls_dh_params);
+void gnutls_certificate_set_rsa_export_params(gnutls_certificate_credentials res, gnutls_rsa_params rsa_params);
+void gnutls_certificate_set_verify_flags(gnutls_certificate_credentials res, unsigned int flags);
+
 
 int gnutls_certificate_set_x509_trust_file( gnutls_certificate_credentials res, const char* CAFILE, 
-	gnutls_x509_certificate_format);
+	gnutls_x509_crt_fmt);
 int gnutls_certificate_set_x509_trust_mem(gnutls_certificate_credentials res, 
-	const gnutls_datum *CA, gnutls_x509_certificate_format);
+	const gnutls_datum *CA, gnutls_x509_crt_fmt);
+
+int gnutls_certificate_set_x509_crl_file(gnutls_certificate_credentials res, 
+		const char *crlfile, gnutls_x509_crt_fmt type);
+int gnutls_certificate_set_x509_crl_mem(gnutls_certificate_credentials res, 
+	const gnutls_datum *CRL, gnutls_x509_crt_fmt type);
 
 int gnutls_certificate_set_x509_key_file( gnutls_certificate_credentials res, 
-	const char *CERTFILE, const char* KEYFILE, gnutls_x509_certificate_format);
+	const char *CERTFILE, const char* KEYFILE, gnutls_x509_crt_fmt);
 int gnutls_certificate_set_x509_key_mem(gnutls_certificate_credentials res, 
 	const gnutls_datum* CERT, const gnutls_datum* KEY,
-	gnutls_x509_certificate_format);
+	gnutls_x509_crt_fmt);
+
+
+/* New functions to allow setting already parsed X.509 stuff.
+ */
+struct gnutls_x509_privkey_int;
+typedef struct gnutls_x509_privkey_int* gnutls_x509_privkey;
+
+struct gnutls_x509_crl_int;
+typedef struct gnutls_x509_crl_int* gnutls_x509_crl;
+
+struct gnutls_x509_crt_int;
+typedef struct gnutls_x509_crt_int* gnutls_x509_crt;
+
+int gnutls_certificate_set_x509_key(gnutls_certificate_credentials res, 
+	gnutls_x509_crt *cert_list, int cert_list_size,
+	gnutls_x509_privkey key);
+int gnutls_certificate_set_x509_trust(gnutls_certificate_credentials res, 
+	gnutls_x509_crt * ca_list, int ca_list_size);
+int gnutls_certificate_set_x509_crl(gnutls_certificate_credentials res, 
+	gnutls_x509_crl* crl_list, int crl_list_size);
+
 
 /* global state functions 
  */
-/* In this version global_init accepts two files (pkix.asn, pkcs1.asn).
- * This will not be the case in the final version. These files 
- * are located in the src/ directory of gnutls distribution.
- */
 int gnutls_global_init(void);
 void gnutls_global_deinit(void);
 
 typedef void* (*gnutls_alloc_function)(size_t);
+typedef void* (*gnutls_calloc_function)(size_t, size_t);
 typedef void (*gnutls_free_function)(void*);
 typedef void* (*gnutls_realloc_function)(void*, size_t);
 
@@ -334,29 +423,52 @@ void gnutls_global_set_mem_functions(
 
 /* For use in callbacks */
 extern gnutls_alloc_function gnutls_malloc;
+extern gnutls_calloc_function gnutls_calloc;
 extern gnutls_free_function gnutls_free;
+extern char* (*gnutls_strdup)( const char*);
 
-typedef void (*gnutls_log_func)( const char*);
+typedef void (*gnutls_log_func)( int, const char*);
 void gnutls_global_set_log_function( gnutls_log_func log_func);
+void gnutls_global_set_log_level( int level);
 
-/* DH params */
-int gnutls_dh_params_set( gnutls_dh_params, gnutls_datum prime, gnutls_datum generator, int bits);
+/* Diffie Hellman parameter handling.
+ */
 int gnutls_dh_params_init( gnutls_dh_params*);
 void gnutls_dh_params_deinit( gnutls_dh_params);
-int gnutls_dh_params_generate( gnutls_datum* prime, gnutls_datum* generator, int bits);
+int gnutls_dh_params_import_raw(gnutls_dh_params dh_params, const gnutls_datum *prime,
+	const gnutls_datum* generator);
+int gnutls_dh_params_import_pkcs3(gnutls_dh_params params, 
+	const gnutls_datum * pkcs3_params, gnutls_x509_crt_fmt format);
+int gnutls_dh_params_generate2(gnutls_dh_params params, int bits);
+int gnutls_dh_params_export_pkcs3( gnutls_dh_params params,
+	gnutls_x509_crt_fmt format, unsigned char* params_data, size_t* params_data_size);
+int gnutls_dh_params_export_raw(gnutls_dh_params params,
+	gnutls_datum * prime, gnutls_datum * generator, unsigned int *bits);
+int gnutls_dh_params_cpy(gnutls_dh_params dst, gnutls_dh_params src);
 
-/* RSA params */
-int gnutls_rsa_params_set(gnutls_rsa_params rsa_params, 
-	gnutls_datum m, gnutls_datum e, gnutls_datum d, 
-	gnutls_datum p, gnutls_datum q, gnutls_datum u,
-	int bits);
-int gnutls_rsa_params_generate(gnutls_datum * m, gnutls_datum *e,
-	gnutls_datum *d, gnutls_datum *p, gnutls_datum* q, 
-	gnutls_datum* u, int bits);
+
+/* RSA params 
+ */
 int gnutls_rsa_params_init(gnutls_rsa_params * rsa_params);
 void gnutls_rsa_params_deinit(gnutls_rsa_params rsa_params);
+int gnutls_rsa_params_cpy(gnutls_rsa_params dst, gnutls_rsa_params src);
+int gnutls_rsa_params_import_raw(gnutls_rsa_params rsa_params, 
+	const gnutls_datum *m, const gnutls_datum *e,
+	const gnutls_datum *d, const gnutls_datum *p, 
+	const gnutls_datum *q, const gnutls_datum *u);
+int gnutls_rsa_params_generate2(gnutls_rsa_params params, int bits);
+int gnutls_rsa_params_export_raw(gnutls_rsa_params params,
+	gnutls_datum * m, gnutls_datum *e,
+	gnutls_datum *d, gnutls_datum *p, gnutls_datum* q, 
+	gnutls_datum* u, unsigned int *bits);
+int gnutls_rsa_params_export_pkcs1( gnutls_rsa_params params,
+        gnutls_x509_crt_fmt format, unsigned char* params_data,
+        size_t* params_data_size);
+int gnutls_rsa_params_import_pkcs1(gnutls_rsa_params params,
+        const gnutls_datum * pkcs1_params, gnutls_x509_crt_fmt format);
 
-
+/* Session stuff
+ */
 typedef ssize_t (*gnutls_pull_func)(gnutls_transport_ptr, void*, size_t);
 typedef ssize_t (*gnutls_push_func)(gnutls_transport_ptr, const void*, size_t);
 void gnutls_transport_set_ptr(gnutls_session session, gnutls_transport_ptr ptr);
@@ -381,116 +493,173 @@ void* gnutls_session_get_ptr(gnutls_session session);
 
 void gnutls_openpgp_send_key(gnutls_session session, gnutls_openpgp_key_status status);
 
-int gnutls_x509_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, char* result, size_t* result_size);
-
+/* fingerprint 
+ * Actually this function returns the hash of the given data.
+ */
+int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, 
+	void* result, size_t* result_size);
 
 #ifndef GNUTLS_UI_H
 # define GNUTLS_UI_H
 
-
-/* Extra definitions */
-
-#define GNUTLS_X509_CN_SIZE 256
-#define GNUTLS_X509_C_SIZE 3
-#define GNUTLS_X509_O_SIZE 256
-#define GNUTLS_X509_OU_SIZE 256
-#define GNUTLS_X509_L_SIZE 256
-#define GNUTLS_X509_S_SIZE 256
-#define GNUTLS_X509_EMAIL_SIZE 256
-
-typedef struct {
-	char common_name[GNUTLS_X509_CN_SIZE];
-	char country[GNUTLS_X509_C_SIZE];
-	char organization[GNUTLS_X509_O_SIZE];
-	char organizational_unit_name[GNUTLS_X509_OU_SIZE];
-	char locality_name[GNUTLS_X509_L_SIZE];
-	char state_or_province_name[GNUTLS_X509_S_SIZE];
-	char email[GNUTLS_X509_EMAIL_SIZE];
-} gnutls_x509_dn;
-
-typedef struct {
-	char name[GNUTLS_X509_CN_SIZE];
-	char email[GNUTLS_X509_CN_SIZE];
-} gnutls_openpgp_name;	
-
 typedef enum gnutls_x509_subject_alt_name {
-	GNUTLS_SAN_DNSNAME=1, GNUTLS_SAN_RFC822NAME,
+	GNUTLS_SAN_DNSNAME = 1, GNUTLS_SAN_RFC822NAME,
 	GNUTLS_SAN_URI, GNUTLS_SAN_IPADDRESS
 } gnutls_x509_subject_alt_name;
 
-# ifdef LIBGNUTLS_VERSION /* These are defined only in gnutls.h */
+# ifdef LIBGNUTLS_VERSION	/* These are defined only in gnutls.h */
+
+/* Callback prototypes for the certificate authentication
+ * callbacks.
+ */
+typedef int gnutls_certificate_client_select_function(gnutls_session,
+      const gnutls_datum * client_cert, int ncerts,
+      const gnutls_datum * req_ca_cert, int nreqs);
+typedef int gnutls_certificate_server_select_function(gnutls_session,
+      const gnutls_datum * server_certs, int ncerts);
+
+
+struct gnutls_openpgp_key_int;
+typedef struct gnutls_openpgp_key_int *gnutls_openpgp_key;
+
+struct gnutls_openpgp_privkey_int;
+typedef struct gnutls_openpgp_privkey_int *gnutls_openpgp_privkey;
+
+typedef struct gnutls_retr_st {
+	gnutls_certificate_type type;
+	union cert {
+		gnutls_x509_crt *x509;
+		gnutls_openpgp_key pgp;
+	} cert;
+	unsigned int ncerts;		/* one for pgp keys */
+
+	union key {
+		gnutls_x509_privkey x509;
+		gnutls_openpgp_privkey pgp;
+	} key;
+
+	unsigned int deinit_all;	/* if non zero all keys will be deinited */
+} gnutls_retr_st;
+
+typedef int gnutls_certificate_client_retrieve_function(gnutls_session,
+	const gnutls_datum* req_ca_rdn, int nreqs,
+	const gnutls_pk_algorithm* pk_algos, int pk_algos_length,
+	gnutls_retr_st *);
+typedef int gnutls_certificate_server_retrieve_function(gnutls_session,
+	gnutls_retr_st *);
 
-typedef int gnutls_certificate_client_select_function(gnutls_session, 
-	const gnutls_datum *, unsigned int, const gnutls_datum *, unsigned int);
-typedef int gnutls_certificate_server_select_function(gnutls_session, 
-	const gnutls_datum *, unsigned int);
 
 /* Functions that allow AUTH_INFO structures handling
  */
 
-gnutls_credentials_type gnutls_auth_get_type( gnutls_session session);
+gnutls_credentials_type gnutls_auth_get_type(gnutls_session session);
 
 /* DH */
 
-void gnutls_dh_set_prime_bits( gnutls_session session, int bits);
-int gnutls_dh_get_prime_bits( gnutls_session);
-int gnutls_dh_get_secret_bits( gnutls_session);
-int gnutls_dh_get_peers_public_bits( gnutls_session);
+void gnutls_dh_set_prime_bits(gnutls_session session, int bits);
+int gnutls_dh_get_prime_bits(gnutls_session);
+int gnutls_dh_get_secret_bits(gnutls_session);
+int gnutls_dh_get_peers_public_bits(gnutls_session);
 
 /* RSA */
 int gnutls_rsa_export_get_modulus_bits(gnutls_session session);
 
 /* X509PKI */
 
-void gnutls_certificate_client_set_select_function( gnutls_session, gnutls_certificate_client_select_function *);
-void gnutls_certificate_server_set_select_function( gnutls_session, gnutls_certificate_server_select_function *);
+#include <gnutls/compat8.h>
 
-void gnutls_certificate_server_set_request( gnutls_session, gnutls_certificate_request);
+/* those should replaced with the functions that follow.
+ */
+void gnutls_certificate_client_set_select_function(gnutls_session,
+	gnutls_certificate_client_select_function *) DEPRECATED;
+void gnutls_certificate_server_set_select_function(gnutls_session,
+	gnutls_certificate_server_select_function *) DEPRECATED;
 
-/* X.509 certificate handling functions */
-int gnutls_x509_certificate_to_xml(const gnutls_datum * cert, gnutls_datum* res, int detail);
+/* These are set on the credentials structure.
+ */
+void gnutls_certificate_client_set_retrieve_function(gnutls_certificate_client_credentials,
+	gnutls_certificate_client_retrieve_function *);
+void gnutls_certificate_server_set_retrieve_function(gnutls_certificate_server_credentials,
+	gnutls_certificate_server_retrieve_function *);
 
-int gnutls_x509_extract_dn( const gnutls_datum*, gnutls_x509_dn*);
-int gnutls_x509_extract_dn_string(const gnutls_datum * idn,
-        char *buf, unsigned int sizeof_buf);
-int gnutls_x509_extract_certificate_dn( const gnutls_datum*, gnutls_x509_dn*);
-int gnutls_x509_extract_certificate_dn_string(char *buf, unsigned int sizeof_buf,
-   const gnutls_datum * cert, int issuer);
-int gnutls_x509_extract_certificate_issuer_dn(  const gnutls_datum*, gnutls_x509_dn *);
-int gnutls_x509_extract_certificate_version( const gnutls_datum*);
-int gnutls_x509_extract_certificate_serial(const gnutls_datum * cert, char* result, int* result_size);
-time_t gnutls_x509_extract_certificate_activation_time( const gnutls_datum*);
-time_t gnutls_x509_extract_certificate_expiration_time( const gnutls_datum*);
-int gnutls_x509_extract_certificate_subject_alt_name( const gnutls_datum*, int seq, char*, int*);
-int gnutls_x509_pkcs7_extract_certificate(const gnutls_datum * pkcs7_struct, int indx, char* certificate, int* certificate_size);
-int gnutls_x509_extract_certificate_pk_algorithm( const gnutls_datum * cert, int* bits);
-int gnutls_x509_extract_certificate_ca_status(const gnutls_datum * cert);
-int gnutls_x509_extract_key_pk_algorithm( const gnutls_datum * key);
+void gnutls_certificate_server_set_request(gnutls_session,
+	gnutls_certificate_request);
 
-int gnutls_x509_verify_certificate( const gnutls_datum* cert_list, int cert_list_length, const gnutls_datum * CA_list, int CA_list_length, const gnutls_datum* CRL_list, int CRL_list_length);
+/* X.509 certificate handling functions 
+ */
 
-int gnutls_x509_check_certificates_hostname(const gnutls_datum * cert,
-                                const char *hostname);
+int gnutls_pkcs3_extract_dh_params(const gnutls_datum * params,
+	gnutls_x509_crt_fmt format, gnutls_datum * prime,
+	gnutls_datum * generator, int *prime_bits);
+int gnutls_pkcs3_export_dh_params(const gnutls_datum * prime,
+	const gnutls_datum * generator, gnutls_x509_crt_fmt format,
+	unsigned char *params_data, int *params_data_size);
 
-/* get data from the session */
-const gnutls_datum* gnutls_certificate_get_peers( gnutls_session, int* list_size);
-const gnutls_datum *gnutls_certificate_get_ours( gnutls_session session);
+/* get data from the session 
+ */
+const gnutls_datum *gnutls_certificate_get_peers(gnutls_session,
+	unsigned int *list_size);
+const gnutls_datum *gnutls_certificate_get_ours(gnutls_session session);
 
 time_t gnutls_certificate_activation_time_peers(gnutls_session session);
 time_t gnutls_certificate_expiration_time_peers(gnutls_session session);
 
-int gnutls_certificate_client_get_request_status(  gnutls_session);
-int gnutls_certificate_verify_peers( gnutls_session);
+int gnutls_certificate_client_get_request_status(gnutls_session);
+int gnutls_certificate_verify_peers(gnutls_session);
 
-int gnutls_pem_base64_encode( const char* header, const gnutls_datum *data, char* result, int* result_size);
-int gnutls_pem_base64_decode( const char* header, const gnutls_datum *b64_data, char* result, int* result_size);
+int gnutls_pem_base64_encode(const char *header, const gnutls_datum * data,
+	char *result, size_t * result_size);
+int gnutls_pem_base64_decode(const char *header,
+	const gnutls_datum * b64_data,
+	unsigned char *result, size_t * result_size);
 
-int gnutls_pem_base64_encode_alloc( const char* header, const gnutls_datum *data, gnutls_datum * result);
-int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_data, gnutls_datum* result);
+int gnutls_pem_base64_encode_alloc(const char *header,
+	const gnutls_datum * data, gnutls_datum * result);
+int gnutls_pem_base64_decode_alloc(const char *header,
+	const gnutls_datum * b64_data, gnutls_datum * result);
 
-# endif /* LIBGNUTLS_VERSION */
+/* key_usage will be an OR of the following values:
+ */
+#define GNUTLS_KEY_DIGITAL_SIGNATURE            128	/* when the key is to be
+							 * used for signing.
+							 */
+#define GNUTLS_KEY_NON_REPUDIATION              64
+#define GNUTLS_KEY_KEY_ENCIPHERMENT             32	/* when the key is to be
+							 * used for encryption.
+							 */
+#define GNUTLS_KEY_DATA_ENCIPHERMENT            16
+#define GNUTLS_KEY_KEY_AGREEMENT                8
+#define GNUTLS_KEY_KEY_CERT_SIGN                4
+#define GNUTLS_KEY_CRL_SIGN                     2
+#define GNUTLS_KEY_ENCIPHER_ONLY                1
+#define GNUTLS_KEY_DECIPHER_ONLY                32768
 
-#endif /* GNUTLS_UI_H */
+typedef struct gnutls_params_st {
+	gnutls_params_type type;
+	union params {
+		gnutls_dh_params dh;
+		gnutls_rsa_params rsa_export;
+	} params;
+	int deinit;
+} gnutls_params_st;
+
+typedef int gnutls_params_function(gnutls_session, gnutls_params_type,
+	gnutls_params_st*);
+
+void gnutls_certificate_set_params_function(gnutls_certificate_credentials res, 
+	gnutls_params_function* func);
+void gnutls_anon_set_params_function(gnutls_certificate_credentials res, 
+	gnutls_params_function* func);
+
+
+
+# endif				/* LIBGNUTLS_VERSION */
+
+#endif				/* GNUTLS_UI_H */
+
+
+#ifndef GNUTLS_ERRORS_IH
+# define GNUTLS_ERRORS_IH
 
 /* Gnutls error codes. The mapping to a TLS alert is also shown in
  * comments.
@@ -519,8 +688,10 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 #define GNUTLS_E_EXPIRED -29
 #define GNUTLS_E_DB_ERROR -30
 #define GNUTLS_E_SRP_PWD_ERROR -31
-#define GNUTLS_E_INSUFICIENT_CREDENTIALS -32
-#define GNUTLS_E_INSUFICIENT_CRED GNUTLS_E_INSUFICIENT_CREDENTIALS
+#define GNUTLS_E_INSUFFICIENT_CREDENTIALS -32
+#define GNUTLS_E_INSUFICIENT_CREDENTIALS GNUTLS_E_INSUFFICIENT_CREDENTIALS /* for backwards compatibility only */
+#define GNUTLS_E_INSUFFICIENT_CRED GNUTLS_E_INSUFFICIENT_CREDENTIALS
+#define GNUTLS_E_INSUFICIENT_CRED GNUTLS_E_INSUFFICIENT_CREDENTIALS /* for backwards compatibility only */
 
 #define GNUTLS_E_HASH_FAILED -33
 #define GNUTLS_E_BASE64_DECODING_ERROR -34
@@ -530,8 +701,6 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 #define GNUTLS_E_GOT_APPLICATION_DATA -38
 #define GNUTLS_E_RECORD_LIMIT_REACHED -39
 #define GNUTLS_E_ENCRYPTION_FAILED -40
-#define GNUTLS_E_CERTIFICATE_ERROR -43
-#define GNUTLS_E_X509_CERTIFICATE_ERROR GNUTLS_E_CERTIFICATE_ERROR
 
 #define GNUTLS_E_PK_ENCRYPTION_FAILED -44
 #define GNUTLS_E_PK_DECRYPTION_FAILED -45
@@ -549,26 +718,11 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 #define GNUTLS_E_PKCS1_WRONG_PAD -57
 #define GNUTLS_E_RECEIVED_ILLEGAL_EXTENSION -58
 #define GNUTLS_E_INTERNAL_ERROR -59
-#define GNUTLS_E_CERTIFICATE_KEY_MISMATCH -60
-#define GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE -61 /* GNUTLS_A_UNSUPPORTED_CERTIFICATE */
-#define GNUTLS_E_X509_UNKNOWN_SAN -62
 #define GNUTLS_E_DH_PRIME_UNACCEPTABLE -63
 #define GNUTLS_E_FILE_ERROR -64
-#define GNUTLS_E_ASN1_ELEMENT_NOT_FOUND -67
-#define GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND -68
-#define GNUTLS_E_ASN1_DER_ERROR -69
-#define GNUTLS_E_ASN1_VALUE_NOT_FOUND -70
-#define GNUTLS_E_ASN1_GENERIC_ERROR -71
-#define GNUTLS_E_ASN1_VALUE_NOT_VALID -72
-#define GNUTLS_E_ASN1_TAG_ERROR -73
-#define GNUTLS_E_ASN1_TAG_IMPLICIT -74
-#define GNUTLS_E_ASN1_TYPE_ANY_ERROR -75
-#define GNUTLS_E_ASN1_SYNTAX_ERROR -76
-#define GNUTLS_E_ASN1_DER_OVERFLOW -77
 #define GNUTLS_E_TOO_MANY_EMPTY_PACKETS -78
-#define GNUTLS_E_OPENPGP_UID_REVOKED -79
 #define GNUTLS_E_UNKNOWN_PK_ALGORITHM -80
-#define GNUTLS_E_OPENPGP_TRUSTDB_VERSION_UNSUPPORTED -81
+
 
 /* returned if libextra functionality was requested but
  * gnutls_global_init_extra() was not called.
@@ -591,11 +745,49 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 
 #define GNUTLS_E_ILLEGAL_SRP_USERNAME -90
 #define GNUTLS_E_SRP_PWD_PARSING_ERROR -91
+#define GNUTLS_E_NO_TEMPORARY_DH_PARAMS -93
 
-#define GNUTLS_E_UNIMPLEMENTED_FEATURE -250
+/* For certificate and key stuff
+ */
+#define GNUTLS_E_ASN1_ELEMENT_NOT_FOUND -67
+#define GNUTLS_E_ASN1_IDENTIFIER_NOT_FOUND -68
+#define GNUTLS_E_ASN1_DER_ERROR -69
+#define GNUTLS_E_ASN1_VALUE_NOT_FOUND -70
+#define GNUTLS_E_ASN1_GENERIC_ERROR -71
+#define GNUTLS_E_ASN1_VALUE_NOT_VALID -72
+#define GNUTLS_E_ASN1_TAG_ERROR -73
+#define GNUTLS_E_ASN1_TAG_IMPLICIT -74
+#define GNUTLS_E_ASN1_TYPE_ANY_ERROR -75
+#define GNUTLS_E_ASN1_SYNTAX_ERROR -76
+#define GNUTLS_E_ASN1_DER_OVERFLOW -77
+#define GNUTLS_E_OPENPGP_TRUSTDB_VERSION_UNSUPPORTED -81
+#define GNUTLS_E_OPENPGP_UID_REVOKED -79
+#define GNUTLS_E_CERTIFICATE_ERROR -43
+#define GNUTLS_E_X509_CERTIFICATE_ERROR GNUTLS_E_CERTIFICATE_ERROR
+#define GNUTLS_E_CERTIFICATE_KEY_MISMATCH -60
+#define GNUTLS_E_UNSUPPORTED_CERTIFICATE_TYPE -61 /* GNUTLS_A_UNSUPPORTED_CERTIFICATE */
+#define GNUTLS_E_X509_UNKNOWN_SAN -62
+#define GNUTLS_E_OPENPGP_FINGERPRINT_UNSUPPORTED -94
+#define GNUTLS_E_X509_UNSUPPORTED_ATTRIBUTE -95
+#define GNUTLS_E_UNKNOWN_HASH_ALGORITHM -96
+#define GNUTLS_E_UNKNOWN_PKCS_CONTENT_TYPE -97
+#define GNUTLS_E_UNKNOWN_PKCS_BAG_TYPE -98
+#define GNUTLS_E_INVALID_PASSWORD -99
+#define GNUTLS_E_MAC_VERIFY_FAILED -100 /* for PKCS #12 MAC */
+
+#define GNUTLS_E_BASE64_ENCODING_ERROR -201
+#define GNUTLS_E_INCOMPATIBLE_GCRYPT_LIBRARY -202
+#define GNUTLS_E_INCOMPATIBLE_LIBTASN1_LIBRARY -203
+
+#define GNUTLS_E_OPENPGP_KEYRING_ERROR -204
+#define GNUTLS_E_X509_UNSUPPORTED_OID -205
+
+#define GNUTLS_E_UNIMPLEMENTED_FEATURE -1250
 
 
 
+#endif /* GNUTLS_ERRORS_IH */
+
 #ifdef __cplusplus
 }
 #endif
